#!/bin/tclsh
#
#  Sitestat Dirscript-2 (rev 11)
#
#--------------------------------------
# setup: 
#  1. copy this file to glftpd's bin dir and chmod to 755
#  2. add this line to glftpd.conf: 
#    pre_dir_check /bin/ss-dir2.tcl
#--------------------------------------
# rule definitions for allowing/disallowing a dir
#
set dircfg "/bin/ss-dir2.cfg"

#--------------------------------------
# enable to prevent duplicate dirs of different case, eg. cd1 & CD1
# 
# note: this preceeds the .cfg rule sets
#
set casechk 1

#--------------------------------------
# set the pattern (regex) the dir name has to follow to be accepted
# 
# note: set * to accept all, this is checked after the rulesets
#
set dirmask {([\w]+[.-_]?)+[-][\w]+}

#--------------------------------------
# set the flags a user must have to be exempt from dirscript rules
# 
# note: likely 1 will do
#
set skipflags "1"

#--------------------------------------
# after much testing it was found many braindead ftp clients 'hang' on
# the output obviously waiting for the expected ftp code in the reply
# and the custom codes had no effect on gl's output, if theres any
# compatibility issues set this to silent mode... 1=silent, 0=normal
# 
set silent "0"


########################################################## begin code.
source /bin/ss-glclib.tcl
if {![info exists glcver] || $glcver != 0.1} {puts "ERROR loading glclib, missing or incorrect version\n" ; exit 0}
set user $env(USER)

proc dircheck {dir} {
	global dircfg dirmask casechk skipflags userpath user
	append userpath "/" $user ; set grp "" ; set pgrp "" ; set pwd [pwd]
	set kmsg "-\[Sitestat-Dirscript\]-----------------------"
	set prepaths [fragr privpath 1]
	if {$prepaths != ""} {
		foreach line $prepaths {
			if [string match [lindex $line 1]/* $pwd] {
###### lappend kmsg "ACCEPTED -> path exempt from dirscript" \n 
##### spitter $kmsg
				exit 0
			}
		}
	}
	set lines [ardw $userpath]	
	if [string equal "e" $lines] {
		lappend kmsg "ERROR processing - $userpath not readable?" \n ; spitter $kmsg
		exit 2
	}
	foreach line [split $lines \n] {
		if [string equal [lindex $line 0] "GROUP"] {append grp " " [lindex $line 1]
		} elseif [string equal [lindex $line 0] "PRIVATE"] {append pgrp " " [lindex $line 1]
		} elseif [string equal [lindex $line 0] "FLAGS"] {set flags [lindex $line 1]}
	}
	set dirlen [string length $dir]
	set of "1" ; set llist "" ; set reas ""
	set dir [string tolower $dir]
	set skipflags "\[$skipflags\]"
	if [regexp $skipflags $flags] {
###### lappend kmsg "ACCEPTED -> user exempt from dirscript" \n 
###### spitter $kmsg
		exit 0
	}
	if {$casechk == 1} {
		set cdz [glob -nocomplain -type d *]
		foreach item [string tolower $cdz] {
			if [string equal $dir $item] {
				lappend kmsg "REJECTED -> exists as \'$item\'" \n 
				spitter $kmsg
				exit 2
			}
		}
	}
	set llist [string tolower [ardw $dircfg]]
	if [string equal "e" $llist] {
		lappend kmsg "ERROR processing - $dircfg missing or unreadable" \n ; spitter $kmsg
		exit 2
	}
	set pgrp [string tolower $pgrp]
	set grp [string tolower $grp]
	set pwd [string tolower $pwd] ; append pwd /
	foreach curline [split $llist \n] {
		set qw [lindex $curline 0]
		if ![string equal "x" [string index $qw 0]] {continue}
		set qc [lindex $curline 1]
		set reas [lindex $curline 2]
		set args [lrange $curline 3 end]
		set rv 0
		foreach arg $args {
			set arg [split $arg ":"]
			set argt [lindex $arg 1]
			switch [lindex $arg 0] {
			 a {if [string match "*$argt*" $dir] {incr rv}}
			 b {if ![string match "*$argt*" $dir] {incr rv}}
			 c {if [string match "$argt*" $dir] {incr rv}}
			 d {if ![string match "$argt*" $dir] {incr rv}}
			 e {if [string match "*$argt" $dir] {incr rv}}
			 f {if ![string match "*$argt" $dir] {incr rv}}
			 g {if {$argt > $dirlen} {incr rv}}
			 h {if {$argt < $dirlen} {incr rv}}
			 i {if [string match $argt $dir] {incr rv}}
			 j {if ![string match $argt $dir] {incr rv}}
			 k {if {[lsearch $grp $argt] != -1} {incr rv}}
			 l {if {[lsearch $grp $argt] == -1} {incr rv}}
			 m {if {[lsearch $pgrp $argt] != -1} {incr rv}}
			 n {if {[lsearch $pgrp $argt] == -1} {incr rv}}
			 o {if [string match "$argt/*" $pwd] {incr rv}}
			 p {if ![string match "$argt/*" $pwd] {incr rv}}
			}
		}
		switch $qw {
			x:0 {if {$qc == $rv} {
#########lappend kmsg "ACCEPTED -> $reas" \n ; spitter $kmsg
				exit 0
			}}
			x:1 {if {$qc == $rv} {lappend kmsg "REJECTED -> $reas" \n ; spitter $kmsg ; exit 2}}
		}
	}
	if {$dirmask != "*"} {
		set dir2 {\.}
		regexp $dirmask $dir dir2
		if {![string equal $dir2 $dir]} {
			lappend kmsg "REJECTED -> name doesn't conform to acceptable pattern" \n ; spitter $kmsg ; exit 2
		}
	}
######lappend kmsg "ACCEPTED -> passed" \n ; spitter $kmsg
	exit 0
}

proc spitter {kmsg} {
  global silent
  if {$silent} {return}
  foreach line $kmsg {puts $line}
}

dircheck [lindex $argv 0]
exit 0
